#!/usr/bin/env python3
# -*- coding: us-ascii -*-
import os
import re
import shutil
import subprocess
import sys

OUTPUT_FILE = '/var/lib/nagios/mdadm.out'
TEMP_FILE = '/tmp/mdadm.out'


def get_devices():
    if os.path.exists('/sbin/mdadm'):
        try:
            devices_raw = subprocess.check_output(
                ['/sbin/mdadm', '--detail', '--scan']
            )
            devices_re = re.compile(r'^ARRAY\s+([^ ]+) ')
            devices = set()
            for line in devices_raw.decode().split('\n'):
                line = line.strip()
                device_re = devices_re.match(line)
                if device_re is not None:
                    devices.add(device_re.group(1))
            return devices
        except subprocess.CalledProcessError as error:
            rc = generate_output(
                'CRITICAL: get_devices error - {}'.format(error)
            )
            if rc:
                sys.exit(0)
    return set()


def generate_output(msg):
    try:
        with open(TEMP_FILE, 'w') as fd:
            fd.write(msg)
        shutil.move(TEMP_FILE, OUTPUT_FILE)
        return True
    except Exception as error:
        print('Unable to generate output file:', error)
        return False


def get_devices_stats(devices):
    mdadm_detail = ['/sbin/mdadm', '--detail']
    mdadm_detail.extend(sorted(devices))

    devices_details_raw = subprocess.check_output(mdadm_detail)

    devices_re = r'^(/\S+):$'
    state_re = r'^\s*State\s+:\s+(.+)\s*$'
    status_re = r'^\s*(Active|Working|Failed|Spare) Devices\s+:\s+(\d+)$'
    rebuild_status_re = r'^\s*Rebuild Status\s+:\s+(\d+%\s+\S+)$'
    removed_re = r'^\s*-\s+0\s+0\s+(\d+)\s+removed$'
    #        4 8 162 3 spare rebuilding /dev/sdk2
    rebuilding_re = r'^\s*\d+\s+\d+\s+\d+\s+\d+\s+\S+\s+rebuilding\s+(\S+)$'

    devices_cre = re.compile(devices_re)
    state_cre = re.compile(state_re)
    status_cre = re.compile(status_re)
    rebuild_status_cre = re.compile(rebuild_status_re)
    removed_cre = re.compile(removed_re)
    rebuilding_cre = re.compile(rebuilding_re)

    device = None
    devices_stats = {}
    for line in devices_details_raw.decode().split('\n'):
        line = line.rstrip()
        m = devices_cre.match(line)
        if m:
            device = m.group(1)
            devices_stats[device] = {
                'stats': {
                    'Active': 0,
                    'Working': 0,
                    'Failed': 0,
                    'Spare': 0,
                },
                'rebuild_status': '',
                'degraded': False,
                'recovering': False,
                'removed': [],
                'rebuilding': [],
            }
            continue

        m = state_cre.match(line)
        if m:
            # format for State line can be "clean" or "clean, degraded" or "active, degraded, rebuilding", etc.
            states = m.group(1).split(", ")
            if 'degraded' in states and device:
                devices_stats[device]['degraded'] = True
            if 'recovering' in states and device:
                devices_stats[device]['recovering'] = True
            continue

        m = status_cre.match(line)
        if m and device:
            devices_stats[device]['stats'][m.group(1)] = int(m.group(2))
            continue

        m = removed_cre.match(line)
        if m and device:
            devices_stats[device]['removed'].append(m.group(1))
            continue

        m = rebuild_status_cre.match(line)
        if m and device:
            devices_stats[device]['rebuild_status'] = m.group(1)
            continue

        m = rebuilding_cre.match(line)
        if m and device:
            devices_stats[device]['rebuilding'].append(m.group(1))
            continue

    return devices_stats


def parse_output():
    devices = get_devices()
    if len(devices) == 0:
        return generate_output('WARNING: unexpectedly checked no devices')

    try:
        devices_stats = get_devices_stats(devices)
    except subprocess.CalledProcessError as error:
        return generate_output(
            'WARNING: error executing mdadm: {}'.format(error)
        )

    msg = []
    critical = False
    warning = False
    for device in devices_stats:
        parts = []
        # Is device degraded?
        if devices_stats[device]['degraded'] and devices_stats[device]['recovering']:
            warning = True
            parts = ['{} recovering'.format(device)]
        elif devices_stats[device]['degraded']:
            critical = True
            parts = ['{} degraded'.format(device)]
        else:
            parts = ['{} ok'.format(device)]

        # If Failed drives are found, list counters (how many?)
        failed_cnt = devices_stats[device]['stats'].get('Failed', 0)
        if failed_cnt > 0:
            critical = True
            dev_stats = [
                '{}[{}]'.format(status, devices_stats[device]['stats'][status])
                for status in sorted(devices_stats[device]['stats'])
            ]
            parts.extend(dev_stats)

        if len(devices_stats[device]['removed']) != 0:
            critical = True
            members = " and ".join(devices_stats[device]['removed'])
            parts.append('RaidDevice(s) {} marked removed'.format(members))

        if len(devices_stats[device]['rebuilding']) != 0:
            rebuilding_members = " ".join(devices_stats[device]['rebuilding'])
            rebuild_status = devices_stats[device]['rebuild_status']
            parts.append('{} rebuilding ({})'.format(rebuilding_members, rebuild_status))

        msg.append(', '.join(parts))

    if critical:
        msg = 'CRITICAL: {}'.format('; '.join(msg))
    elif warning:
        msg = 'WARNING: {}'.format('; '.join(msg))
    else:
        msg = 'OK: {}'.format('; '.join(msg))
    return generate_output(msg)


if __name__ == '__main__':
    parse_output()
